/************************************************************************
** MODULE INFORMATION **
************************
** FILE NAME:          netif.c
** SYSTEM NAME:        Netmon
** TYPE NAME:          network interface
** ORIGINAL AUTHOR(S): L. Th. Weerpas & W. Haverhals
** VERSION NUMBER:     1.0
** CREATION DATE:      1992/6/9
**
** DESCRIPTION:        network interval stats retrieval
*************************************************************************
** CHANGES INFORMATION **
*************************
** REVISION:           $Revision$
** WORKFILE:           $Workfile$
** LOGINFO:            $Log$
*************************************************************************
** COPYRIGHT           (c) 1992
**                     L. Th. Weerpas & W. Haverhals
**                     All rights reserved.
**
** NOTICE              Redistribution  and use in source and binary forms
**                     are  permitted  provided  that the above copyright
**                     notice  and  this  paragraph are duplicated in all
**                     such forms and that any documentation, advertising
**                     materials,  and  other  materials  related to such
**                     distribution and use acknowledge that the software
**                     was  developed  by  above  mentioned  authors. The
**                     name(s)  of the authors may not be used to endorse
**                     or  promote  products  derived  from this software
**                     without  specific  prior  written permission.
**                     THIS  SOFTWARE  IS  PROVIDED ``AS IS'' AND WITHOUT
**                     ANY  EXPRESS  OR  IMPLIED  WARRANTIES,  INCLUDING,
**                     WITHOUT  LIMITATION,  THE  IMPLIED  WARRANTIES  OF
**                     MERCHANTIBILITY   AND  FITNESS  FOR  A  PARTICULAR
**                     PURPOSE.
************************************************************************/

#define _NETIF_C

#include "netmon.h"
#include "netif.h"

/*
 * GLOBALS
 */

static int       nifs;                            /* nr of interfaces */
static INTERFACE ifs[ IF_MAX ];                   /* if ctrl blocks   */

/*
 * kernel access
 */

static KVMLIST symlist[] =
{
    { "_ifnet" },
    { NULL }
};
#define NSYM  (int)(sizeof(symlist)/sizeof( KVMLIST ))

static KVMOFST ofst_ifnet;

/*
 * screen access
 */

static MODTEXT iftext[] =
{
    {  0,  0, "ACTIVE NETWORK INTERFACES" }
};
#define NTEXT (sizeof(iftext)/sizeof(MODTEXT))

#define LXT (17)
#define LX  (30)

#define LYT (15)
#define LYA (2)
#define LY  (LYT+IFADDR_MAX*LYA)

static MODTEXT ifbody[] =
{
    {  0, 0, "IF name" },                                 /* static part   */
    {  1, 0, "IF sub-unit" },
    {  2, 0, "max xmit unit" },
    {  3, 0, "flags" },                                   /* volatile part */
    {  4, 0, "rqsts for promisc" },
    {  5, 0, "metric" },
    {  6, 0, "queue length" },
    {  7, 0, "queue max length" },
    {  8, 0, "queue drops" },
    {  9, 0, "packets in" },
    { 10, 0, "errors in" },
    { 11, 0, "packets out" },
    { 12, 0, "errors out" },
    { 13, 0, "collisions" }
};
#define NBODY (sizeof(ifbody)/sizeof(MODTEXT))

static MODTEXT ifaddr[] =
{
    {  0, 0, "IF address family" },
    {  1, 0, "IF address" }
};
#define NADDR (sizeof(ifaddr)/sizeof(MODTEXT))

/* ----------------------------- screen layout ---------------------------------

IF NAME            abcdefghi  IF NAME            abcdefghi
sub-unit          1234567890  sub-unit          1234567890
max xmit unit     1234567890  max xmit unit     1234567890
flags             1234567890  flags             1234567890
rqsts for promisc 1234567890  rqsts for promisc 1234567890
metric            1234567890  metric            1234567890
queue length      1234567890  queue length      1234567890
queue max length  1234567890  queue max length  1234567890
queue drops       1234567890  queue drops       1234567890
RECEIVED packets  1234567890  RECEIVED packets  1234567890
         errors   1234567890           errors   1234567890
SENT     packets  1234567890  SENT     packets  1234567890
         errors   1234567890           errors   1234567890
COLLISIONS        1234567890  COLLISIONS        1234567890
IF adrs family       AF_INET  IF adrs family       AF_INET
IF address   123.456.789.abc  IF address   123.456.789.abc
IF adrs family       AF_INET  IF adrs family       AF_INET
IF address   123.456.789.abc  IF address   123.456.789.abc
IF adrs family       AF_INET  IF adrs family       AF_INET
IF address   123.456.789.abc  IF address   123.456.789.abc
----------------------------------------------------------------------------- */

MODFUNC modif = { IfInit, IfText, IfUpdate, 1+IF_MAX*LX, LY+1 };

/*
 * local prototypes
 */

void IfPutAdrs( int x, int y, IFADR *ifadr );

void IfCnt1( int x, int y, u_long *new );
void IfCnt2( int x, int y, u_long *new, u_long *old);

#define O1(_dx,_dy,_fld) IfCnt1(x+_dx,y+_dy, &pnew->_fld)
#define O2(_dx,_dy,_fld) IfCnt2(x+_dx,y+_dy, &pnew->_fld, &pold->_fld)

/*****************************************************************
** NAME:        IfInit                                      NETMON
** SYNOPSIS:    void IfInit
**              (
**                void
**              )
** RELATED TO:  If..()
** DESCRIPTION: initialise if stats
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void IfInit( void )
{
    int        i,j;
    int        r;
    INTERFACE *pifs;
    IFNET     *pifnet;
    IFADR     *pifa;
    KVMOFST    cur_ofst;
    KVMOFST    ofst_addr;

    /*
     * init
     */

    memset( &ifs, 0, sizeof( ifs ));

    /*
     * locate the wanted interface link root
     */

    KvmNlist( symlist, NSYM, __FILE__ );
    ofst_ifnet = symlist[0].n_value;

    /*
     * initialise the IF DB for Netmon
     */

    cur_ofst = ofst_ifnet;
    nifs     = 0;

    KvmRead3( (KVMOFST) ofst_ifnet, &cur_ofst, sizeof( IFNET *));

    for ( i=0; i < IF_MAX && cur_ofst; i++ )
    {
        pifs = &ifs[ i ];

        /*
         * init [i]
         */

        pifs->offset = cur_ofst;        /* image offset of this IF in kvm */
        pifs->pold   = &pifs->stat1;
        pifs->pnew   = &pifs->stat2;

        /*
         * read the ifnet image into .pold
         */

        pifnet = pifs->pold;
        r = KvmRead3( cur_ofst, pifnet, sizeof( IFNET ));   /* read ifnet */
        if ( r < 0 )
            break;            /* read error */

        /*
         * read the IF name
         */

        r = KvmRead3( (KVMOFST) pifnet->if_name, pifs->name, IFNAMSIZ );
        if ( r < 0 )
            break;            /* read error */
        pifs->name[ IFNAMSIZ ] = '\0';

        /*
         * read the IF adresses
         */

        ofst_addr = (KVMOFST) pifnet->if_addrlist;             /* first entry */
        pifs->naddr = 0;

        for ( j = 0; j < IFADDR_MAX && ofst_addr; j++ )
        {
            pifa = &pifs->addr[ j ];

            /*
             * read 1 address
             */

            r = KvmRead3( ofst_addr, pifa, sizeof( IFADR ));
            if ( r < 0 )
                break;            /* read error */

            /*
             * all done, to next address
             */

            ofst_addr = (KVMOFST) pifa->ifa_next;
            pifs->naddr++;
        }

        /*
         * all done, to next IF
         */

        cur_ofst = (KVMOFST) pifnet->if_next;
        nifs++;
    }
}

/*****************************************************************
** NAME:        IfText                                      NETMON
** SYNOPSIS:    void IfText
**              (
**                int x,      relative colum origin
**                int y,      relative row   origin
**              )
** RELATED TO:  If..()
** DESCRIPTION: display if texts
** REMARKS:     - depends on nr of interfaces
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void IfText( int x, int y )
{
    int        i;
    int        j;
    int        r;             /* row    */
    int        c;             /* column */
    INTERFACE *pifs;
    IFNET     *pifnet;

    if ( nifs == 0 )
        return;
    TermValue( iftext, x, y, NTEXT );                       /* header       */

    for ( i = 0; i < nifs; i++ )
    {
        r = y + 1;                                          /* new row      */
        c = x + i * LX;                                     /* new column   */
        pifs = &ifs[ i ];
        pifnet = pifs->pold;
        TermValue( ifbody, c, r, NBODY );                   /* IF body text */

        /*
         * IF adresses
         */

        r += LYT;
        for ( j = 0; j < pifs->naddr; j++ )
        {
            TermValue( ifaddr, c, r, NADDR );               /* IF adrs text */
            IfPutAdrs( c + LXT, r, pifs->addr );            /* and value    */
            r += LYA;
        }
    }
}

/*****************************************************************
** NAME:        IfUpdate                                    NETMON
** SYNOPSIS:    void IfUpdate
**              (
**                int x,      relative colum origin
**                int y,      relative row   origin
**              )
** RELATED TO:  If..()
** DESCRIPTION: display if values
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void IfUpdate( int x, int y )
{
    int        i;
    int        j;
    int        r;             /* row    */
    int        c;             /* column */

    INTERFACE *pifs;
    IFNET     *pifnew;
    IFNET     *swap;
    IFADR     *pifa;

    for ( i = 0; i < nifs; i++ )
    {
        c = x + LXT + i*LX;
        r = y+1;
        pifs = &ifs[ i ];

        swap       = pifs->pold;
        pifs->pold = pifs->pnew;
        pifs->pnew = swap;
        pifnew     = pifs->pnew;

        /*
         * read the new IFNET values
         */

        KvmRead( pifs->offset, pifnew, sizeof( IFNET ));

        /* static values */

        tprintf( c, r   , " %10.10s",  pifs->name );      /* IF name      */
        tprintf( c, r+1 , " % 10d",    pifnew->if_unit ); /* IF unit      */
        tprintf( c, r+2 , " % 10d",    pifnew->if_mtu  ); /* IF mtu       */

        /*
         * print the new values
         */

        tprintf( c, r+3,  " % 10lx", (u_long) pifnew->if_flags   );
        tprintf( c, r+4,  " % 10lu", (u_long) pifnew->if_promisc );
        tprintf( c, r+5,  " % 10lu", (u_long) pifnew->if_metric  );
        tprintf( c, r+6,  " % 10lu", (u_long) pifnew->if_snd.ifq_len );
        tprintf( c, r+7,  " % 10lu", (u_long) pifnew->if_snd.ifq_maxlen );
        tprintf( c, r+8,  " % 10lu", (u_long) pifnew->if_snd.ifq_drops );
        tprintf( c, r+9,  " % 10lu", (u_long) pifnew->if_ipackets );
        tprintf( c, r+10, " % 10lu", (u_long) pifnew->if_ierrors  );
        tprintf( c, r+11, " % 10lu", (u_long) pifnew->if_opackets );
        tprintf( c, r+12, " % 10lu", (u_long) pifnew->if_oerrors );
        tprintf( c, r+13, " % 10lu", (u_long) pifnew->if_collisions );
    }
}

/*****************************************************************
** NAME:        IfPutAdrs                                   NETMON
** SYNOPSIS:    void IfPutAdrs
**              (
**                int x,      absolute colum origin
**                int y,      absolute row   origin
**              )
** RELATED TO:  If..()
** DESCRIPTION: display if values
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

#define CLA (x-7)
#define RWA (y+1)

void IfPutAdrs( int x, int y, IFADR *ifadr )
{
    struct sockaddr *sa;

    if ( ifadr == NULL )
        return;

    /*
     * decode the family addresses, only AF_INET is understood!!
     */

    sa = &ifadr->ifa_addr;
    switch ( sa->sa_family )
    {
        case AF_UNSPEC:
            tprintf( x, y,     " %10.10s", "AF_UNSPEC" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_UNIX:
            tprintf( x, y,     " %10.10s", "AF_UNIX" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_INET:
        {
            struct sockaddr_in *sai = (struct sockaddr_in *) sa;
            tprintf( x, y,     " %10.10s", "AF_INET" );
            tprintf( CLA, RWA, " %17.17s", inet_ntoa( sai->sin_addr ));
            break;
        }
        case AF_IMPLINK:
            tprintf( x, y,     " %10.10s", "AF_IMPLINK" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_PUP:
            tprintf( x, y,     " %10.10s", "AF_PUP" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_CHAOS:
            tprintf( x, y,     " %10.10s", "AF_CHAOS" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_NS:
            tprintf( x, y,     " %10.10s", "AF_NS" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_NBS:
            tprintf( x, y,     " %10.10s", "AF_NBS" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_ECMA:
            tprintf( x, y,     " %10.10s", "AF_ECMA" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_DATAKIT:
            tprintf( x, y,     " %10.10s", "AF_DATAKIT" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_CCITT:
            tprintf( x, y,     " %10.10s", "AF_CCITT" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_SNA:
            tprintf( x, y,     " %10.10s", "AF_SNA" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_DECnet:
            tprintf( x, y,     " %10.10s", "AF_DECnet" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_DLI:
            tprintf( x, y,     " %10.10s", "AF_DLI" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_LAT:
            tprintf( x, y,     " %10.10s", "AF_LAT" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_HYLINK:
            tprintf( x, y,     " %10.10s", "AF_HYLINK" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_APPLETALK:
            tprintf( x, y,     " %10.10s", "AF_APPLETALK" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_NIT:
            tprintf( x, y,     " %10.10s", "AF_NIT" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_802:
            tprintf( x, y,     " %10.10s", "AF_802" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_OSI:
            tprintf( x, y,     " %10.10s", "AF_OSI" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_X25:
            tprintf( x, y,     " %10.10s", "AF_X25" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_OSINET:
            tprintf( x, y,     " %10.10s", "AF_OSINET" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        case AF_GOSIP:
            tprintf( x, y,     " %10.10s", "AF_GOSIP" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
        default:
            tprintf( x, y,     " %10.10s", "???" );
            tprintf( CLA, RWA, " %17.17s", "-" );
            break;
    }
}

/* end of netif.c */


