/************************************************************************
** MODULE INFORMATION **
************************
** FILE NAME:          netinfo.c
** SYSTEM NAME:        Netmon
** TYPE NAME:          Netmon info
** ORIGINAL AUTHOR(S): L. Th. Weerpas & W. Haverhals
** VERSION NUMBER:     1.0
** CREATION DATE:      1992/6/9
**
** DESCRIPTION:        Netmon info in separate screen (!?)
*************************************************************************
** CHANGES INFORMATION **
*************************
** REVISION:           $Revision$
** WORKFILE:           $Workfile$
** LOGINFO:            $Log$
*************************************************************************
** COPYRIGHT           (c) 1992
**                     L. Th. Weerpas & W. Haverhals
**                     All rights reserved.
**
** NOTICE              Redistribution  and use in source and binary forms
**                     are  permitted  provided  that the above copyright
**                     notice  and  this  paragraph are duplicated in all
**                     such forms and that any documentation, advertising
**                     materials,  and  other  materials  related to such
**                     distribution and use acknowledge that the software
**                     was  developed  by  above  mentioned  authors. The
**                     name(s)  of the authors may not be used to endorse
**                     or  promote  products  derived  from this software
**                     without  specific  prior  written permission.
**                     THIS  SOFTWARE  IS  PROVIDED ``AS IS'' AND WITHOUT
**                     ANY  EXPRESS  OR  IMPLIED  WARRANTIES,  INCLUDING,
**                     WITHOUT  LIMITATION,  THE  IMPLIED  WARRANTIES  OF
**                     MERCHANTIBILITY   AND  FITNESS  FOR  A  PARTICULAR
**                     PURPOSE.
************************************************************************/

#define _NETINFO_C

#include "netmon.h"
#include "nettcpso.h"	   
#include "netnames.h"
#include "netinfo.h"

/*
 * GLOBALS
 */

/*
 * screen access
 */

MODFUNC modinfo = { InfoInit, InfoText, InfoUpdate, 73, 8 };
MODTEXT infotext[] =
{
    { 0,  0, "MRU_CACHE____________  "
             "LENGTH  "
             "#HITS____  "
             "#MISSES__  "
             "#UNRESOLV  "
             "#TO_FRONT"  },
    { 1,  0, "Internet addresses" },
    { 2,  0, "TCP ports" },
    { 3,  0, "UDP ports" },

    { 5,  0, "KVM_BYTES_READ__  KVM_BADREAD  CHARS_TO_TERM__" }
};
#define NTEXT (sizeof(infotext)/sizeof(MODTEXT))

/* -----------------------------------------------------------------------------
MRU_CACHE____________  LENGTH  #HITS____  #MISSES__  #UNRESOLV  #TO_FRONT
Internet addresses     12345   123456789  123456789  123456789  123456789
TCP ports              12345   123456789  123456789  123456789  123456789
UDP ports              12345   123456789  123456789  123456789  123456789

KVM_BYTES_READ__  KVM_BADREAD  CHARS_TO_TERM__
 123456789012345   123456789   123456789012345
----------------------------------------------------------------------------- */

/*
 * local prototypes
 */

void InfoInit( void );
void InfoText( int x, int y );
void InfoUpdate( int x, int y );

/*
 * terminal output macros
 */

#define OS(_dx,_dy,_par) tprintf( x+_dx, y+_dy, " % 5u", _par)
#define OU(_dx,_dy,_par) tprintf( x+_dx, y+_dy, " % 9lu", _par)
#define OD(_dx,_dy,_par) tprintf( x+_dx, y+_dy, " % 15.0f", _par)

/*****************************************************************
** NAME:        InfoInit                                     NETMON
** SYNOPSIS:    void InfoInit
**              (
**                void
**              )
** RELATED TO:  Info.()
** DESCRIPTION: initialise netmon stats
** REMARKS:     - dummy
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void InfoInit( void )
{
}

/*****************************************************************
** NAME:        InfoText                                   NETMON
** SYNOPSIS:    void InfoText
**              (
**                int x,      absolute column origin
**                int y       absolute row    origin
**              )
** RELATED TO:  Info..()
** DESCRIPTION: print NNetmon Info exts
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void InfoText( int x, int y )
{
    TermValue( infotext, x, y, NTEXT );
}

/*****************************************************************
** NAME:        InfoUpdate                                   NETMON
** SYNOPSIS:    void InfoUpdate
**              (
**                int x,      absolute column origin
**                int y       absolute row    origin
**              )
** RELATED TO:  Info..()
** DESCRIPTION: display Info values
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/
		   			    
void InfoUpdate( int x, int y )
{
    /*
     * netnames.c stats
     */

    OS( 22, 1, mru.len    );
    OU( 30, 1, mru.hits   );
    OU( 41, 1, mru.misses );
    OU( 52, 1, mru.nores  );
    OU( 63, 1, mru.fronts );

    OS( 22, 2, mru_tcp.len    );
    OU( 30, 2, mru_tcp.hits   );
    OU( 41, 2, mru_tcp.misses );
    OU( 52, 2, mru_tcp.nores  );
    OU( 63, 2, mru_tcp.fronts );

    OS( 22, 3, mru_udp.len    );
    OU( 30, 3, mru_udp.hits   );
    OU( 41, 3, mru_udp.misses );
    OU( 52, 3, mru_udp.nores  );
    OU( 63, 3, mru_udp.fronts );

    /*
     * netkern.c stats
     */

    OD(  0, 6, kvmreadbytes );
    OU( 19, 6, kvmreaderrors );

    /*
     * netterm.c stats
     */

    OD( 30,6, termcharcount );
}

/* end of netinfo.c */


