/************************************************************************
** MODULE INFORMATION **
************************
** FILE NAME:          netmon.c
** SYSTEM NAME:        Netmon
** TYPE NAME:          main body
** ORIGINAL AUTHOR(S): L. Th. Weerpas & W. Haverhals
** VERSION NUMBER:     1.0
** CREATION DATE:      1992/6/9
**
** DESCRIPTION:        main body which invokes all net modules
*************************************************************************
** CHANGES INFORMATION **
*************************
** REVISION:           $Revision$
** WORKFILE:           $Workfile$
** LOGINFO:            $Log$
*************************************************************************
** COPYRIGHT           (c) 1992
**                     L. Th. Weerpas & W. Haverhals
**                     All rights reserved.
**
** NOTICE              Redistribution  and use in source and binary forms
**                     are  permitted  provided  that the above copyright
**                     notice  and  this  paragraph are duplicated in all
**                     such forms and that any documentation, advertising
**                     materials,  and  other  materials  related to such
**                     distribution and use acknowledge that the software
**                     was  developed  by  above  mentioned  authors. The
**                     name(s)  of the authors may not be used to endorse
**                     or  promote  products  derived  from this software
**                     without  specific  prior  written permission.
**                     THIS  SOFTWARE  IS  PROVIDED ``AS IS'' AND WITHOUT
**                     ANY  EXPRESS  OR  IMPLIED  WARRANTIES,  INCLUDING,
**                     WITHOUT  LIMITATION,  THE  IMPLIED  WARRANTIES  OF
**                     MERCHANTIBILITY   AND  FITNESS  FOR  A  PARTICULAR
**                     PURPOSE.
************************************************************************/

#include "netmon.h"
#include "netif.h"
#include "netip.h"
#include "nettcp.h"
#include "nettcpso.h"
#include "netudp.h"
#include "netudpso.h"
#include "neticmp.h"
#include "netmem.h"
#include "netinfo.h"

/*
 * local prototypes
 */

void BadUsage ( int   exitcode );
int  OptSearch( char *option );
void SysOpen  ( void );
void SysStats ( void );
int  SysStop  ( int signum );
void MonInit  ( void );
void MonText  ( int  x, int y );
void MonUpdate( int  x, int y );

/*
 * defines and typedefs
 */

#define MIN_PERIOD      (250)
#define MAX_PERIOD      (60000)

#define SHOW_ON         (1)
#define SHOW_OFF        (0)

#define MON_MON         (0)
#define MON_INFO        (1)
#define MON_MEM         (2)
#define MON_IF          (3)
#define MON_IP          (4)
#define MON_ICMP        (5)
#define MON_UDP         (6)
#define MON_UDPCON      (7)
#define MON_TCP         (8)
#define MON_TCPCON      (9)
#define MON_MAX         (10)

#define OPTNAME_ALL     "-ALL"
#define OPTNAME_INFO    "-INFO"
#define OPTNAME_MEM     "-MEM"
#define OPTNAME_IF      "-IF"
#define OPTNAME_IP      "-IP"
#define OPTNAME_ICMP    "-ICMP"
#define OPTNAME_UDP     "-UDP"
#define OPTNAME_UDPCON  "-UDPSOCK"
#define OPTNAME_TCP     "-TCP"
#define OPTNAME_TCPCON  "-TCPSOCK"

/*
 * GLOBALS
 */

char *master  = "xnetmon";
char *slave   = "netmon";
char *version = "XNetmon v1.0 by Weerpas & Haverhals";

char *optlist[ MON_MAX ] =
{
    OPTNAME_ALL,
    OPTNAME_INFO,
    OPTNAME_MEM,
    OPTNAME_IF,
    OPTNAME_IP,
    OPTNAME_ICMP,
    OPTNAME_UDP,
    OPTNAME_UDPCON,
    OPTNAME_TCP,
    OPTNAME_TCPCON
};
#define NOPT ((int)(sizeof( optlist )/sizeof( char * )))

#define NHDR (55 )
MODTEXT montext[] =
{
    { 0,  0, "  NetMon v1.0, (c)'92 by W.A.P.Haverhals & L.Th.Weerpas" },
    { 1,  0, "Uptime:              [s]      Interval:        [s]     " },
    { 2,  0, "-------------- TU Delft, the DNPAP group --------------" }
};
#define NTEXT (sizeof(montext)/sizeof(MODTEXT))
MODFUNC modmon  = { MonInit, MonText, MonUpdate, NHDR, NTEXT+1 };

/*
 * miscellaneous
 */

short          active      = 0;
char          *prog        = NULL;
unsigned long  displaywait = -1L;

#define MOD_INIT( _n_, _p_, _s_ ) mon[_n_].func=_p_; mon[_n_].display=_s_

MON_MODULE mon[ MON_MAX ];

/*****************************************************************
** NAME:        main                                        NETMON
** SYNOPSIS:    void main
**              (
**                int argc,     nr of arguments
**                char *argv[]  char ptrs to arguments
**              )
** RELATED TO:  .
** DESCRIPTION: main bidy of netmon
** REMARKS:     use as follows:
**              invocation: <prog> [options] <interval in milsec>
**              options:    -ALL      all modules
**                          -MEM      mbuf      stats
**                          -INFO     statistics
**                          -IF       interface stats
**                          -IP       IP        stats
**                          -ICMP     ICMP      stats
**                          -UDP      UDP       stats
**                          -UDPSOCK  UDP       sockets
**                          -TCP      TCP       stats
**                          -TCPSOCK  TCP       sockets
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void main( int argc, char *argv[] )
{
    int   i;
    int   rows, r;
    int   cols, c;
    char *opt_1;
    char *opt_i;
    char  cmd[ 512 ];

    prog = argv[0];

    /*
     * set the defaults
     */

    MOD_INIT( MON_MON,    &modmon  , SHOW_ON  );
    MOD_INIT( MON_INFO,   &modinfo , SHOW_OFF );
    MOD_INIT( MON_MEM,    &modmem  , SHOW_OFF );
    MOD_INIT( MON_IF,     &modif   , SHOW_OFF );
    MOD_INIT( MON_IP,     &modip   , SHOW_OFF );
    MOD_INIT( MON_ICMP,   &modicmp , SHOW_OFF );
    MOD_INIT( MON_UDP,    &modudp  , SHOW_OFF );
    MOD_INIT( MON_UDPCON, &modudpso, SHOW_OFF );
    MOD_INIT( MON_TCP,    &modtcp  , SHOW_OFF );
    MOD_INIT( MON_TCPCON, &modtcpso, SHOW_OFF );

    /*
     * decode argu's
     */

    if ( argc < 2 )
        BadUsage( 1 );

    for ( i=1; i<argc; i++ )
    {
       OptSearch( argv[i] );
    }

    /*
     * start the xterms
     */

    if ( strcmp( prog, master ) == 0 )
    {
        /*
         * add the header
         */

        c = mon[ 0 ].func->xdim;
        r = mon[ 0 ].func->ydim;

        /*
         * is -INFO specified ?
         */

        if ( mon[ MON_INFO].display == SHOW_ON )
        {
            opt_1 = optlist[ MON_INFO ];
            if ( c < mon[ MON_INFO].func->xdim )
                c = mon[ MON_INFO].func->xdim;
            r += mon[ MON_INFO].func->ydim;
        }
        else
        {
            opt_1 = " ";
        }

        /*
         * start the xterms with possibly -INFO on
         */

        for ( i = MON_INFO + 1; i < MON_MAX; i++ )
        {
            rows = r;
            cols = c;
            if ( mon[ i ].display == SHOW_ON )
            {
                if ( cols < mon[ i ].func->xdim )
                    cols = mon[ i ].func->xdim;
                rows += mon[ i ].func->ydim;
                opt_i = optlist[ i ];
                sprintf( cmd,
                         "xterm "
                         "-cr red "               /* text cursor color        */
                         "-fg black "             /* text foreground color    */
                         "-bg white "             /* text background color    */
                         "+sb "                   /* scroll bar : off         */
                         "-fn 6x10 "              /* font: 6x10 fixed         */
                         "-geometry %ldx%ld-2+2 " /* screen dim. and pos.     */
                         "-title \"%s %s\" "      /* xterm title              */
                         "-e %s %s %s %ld &",     /* our command, in bg       */
                         cols, rows,
                         opt_i, version,
                         slave, opt_1, opt_i, displaywait );

                printf( "Executing: %s\n" , cmd );
                system( cmd );
            }
        }
        exit( 0 );
    }

    /*
     * start the snapshot viewer
     */

    if ( strcmp( prog, slave ) == 0 )
    {
        SysOpen();
        SysStats();
        SysClose( 0 );
    }

    /*
     * bad program name!
     */

    fprintf( stderr, "\nProgram name should be '%s' or '%s'\n", slave, master );
    exit( 2 );
}

/*****************************************************************
** NAME:        SysOpen                                     NETMON
** SYNOPSIS:    void SysOpen
**              (
**                void
**              )
** RELATED TO:  Sys..()
** DESCRIPTION: open all resources: general initialisation
** REMARKS:     - opens: kmem/terminal/time
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void SysOpen( void )
{
    int         i;
    int         cols;
    int         rows;
    MON_MODULE *pmon;

    KvmOpen();
    TermOpen();
    TimeOpen();
    NetNameOpen();


    /*
     * add the signal handler
     */

    signal( SIGINT,  SysStop );
    signal( SIGABRT, SysStop );
    signal( SIGTERM, SysStop );

    /*
     * calculate max screen dimensions and test with current dimensions
     */

    cols = 0;
    rows = 0;
    for ( i=0; i<MON_MAX; i++ )
    {
        pmon = &mon[i];
        if ( pmon->display == SHOW_ON )
        {
            rows += pmon->func->ydim;
            if ( cols < pmon->func->xdim )
                cols = pmon->func->xdim;
        }
    }
    if ( rows > tdim_height || cols > tdim_width )
    {
        fprintf( stderr, "%s: detected TERMINAL error:\n\n"
                         "  screen is too small (%dx%d)\n"
                         "  must be             (%dx%d)\n\n",
                         prog,\
                         tdim_width, tdim_height,\
                         cols, rows );
        SysClose( 1 );
    }

    /*
     * initialise MODULEs and print static texts
     */

    rows = 0;
    for ( i=0; i<MON_MAX; i++ )
    {
        pmon = &mon[i];
        if ( pmon->display == SHOW_ON )
        {
            DO_INIT( pmon );
            DO_TEXT( pmon, 0, rows );
            rows += pmon->func->ydim;
        }
    }
    usleep( 1000L );
}

/*****************************************************************
** NAME:        SysOpen                                     NETMON
** SYNOPSIS:    void SysOpen
**              (
**                int signum    the calling signal
**              )
** RELATED TO:  Sys..()
** DESCRIPTION: stop NetMon gracefully
** REMARKS:     - SIGNAL handler
** RETURN TYPE: int
** EXIT STATUS: -
*****************************************************************/

int SysStop( int signum )
{
   signum = 0;
   active = 0;
}

/*****************************************************************
** NAME:        SysStats                                    NETMON
** SYNOPSIS:    void SysStats
**              (
**                void
**              )
** RELATED TO:  Sys..()
** DESCRIPTION: display selected stats
** REMARKS:     - stat retriever and displayer
** RETURN TYPE: int
** EXIT STATUS: -
*****************************************************************/

void SysStats( void )
{
    int         i;
    int         yoffs;
    MON_MODULE *pmon;

    active = 1;
    while (active)
    {
        TimeGet();
        yoffs=0;
        for ( i=0; i<MON_MAX; i++ )
        {
            pmon  = &mon[i];
            if ( pmon->display == SHOW_ON )
            {
                DO_UPDATE( pmon, 0, yoffs );
                yoffs += pmon->func->ydim;
            }
        }
        TermSetHome();
        fflush( stdout );
        usleep( 1000*displaywait );
    }
}

/*****************************************************************
** NAME:        SysClose                                    NETMON
** SYNOPSIS:    void SysClose
**              (
**                int exitcode  return value of netmon executable
**              )
** RELATED TO:  Sys..()
** DESCRIPTION: close all resources
** REMARKS:     - general clean-up
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void SysClose( int exitcode )
{
    TermClose();
    KvmClose();
    fflush( stdout );
    exit( exitcode );
}

/*****************************************************************
** NAME:        BadUsage                                    NETMON
** SYNOPSIS:    void BadUsage
**              (
**                int exitcode  return value of netmon executable
**              )
** RELATED TO:  main()
** DESCRIPTION: show usage on invalid command line
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void BadUsage( int exitcode )
{
    fprintf( stderr,
             "Incorrect commandline syntax, use as follows\n\n"
             "Invocation: %s [options] <%ld..%ld millisecs>\n\n"
             "Options:    %8s  all       stats\n"
             "            %8s  Some statistics\n"
             "            %8s  mbuf      stats\n"
             "            %8s  interface stats\n"
             "            %8s  IP        stats\n"
             "            %8s  ICMP      stats\n"
             "            %8s  UDP       stats\n"
             "            %8s  UDP       sockets\n"
             "            %8s  TCP       stats\n"
             "            %8s  TCP       sockets\n"
             "\nExample: xnetmon -ip 1000\n\n",
             prog,
             (long) MIN_PERIOD, (long) MAX_PERIOD,
             optlist[ MON_MON ],
             optlist[ MON_INFO ],
             optlist[ MON_MEM ],
             optlist[ MON_IF ],
             optlist[ MON_IP ],
             optlist[ MON_ICMP ],
             optlist[ MON_UDP ],
             optlist[ MON_UDPCON ],
             optlist[ MON_TCP ],
             optlist[ MON_TCPCON ] );
    exit( exitcode );
}

/*****************************************************************
** NAME:        OptSearch                                   NETMON
** SYNOPSIS:    int OptSearch
**              (
**                char *option  ptr to cmd-line argument
**              )
** RELATED TO:  main()
** DESCRIPTION: decode an option string
** REMARKS:     -
** RETURN TYPE: int
** EXIT STATUS: 0:  found an option
**              -1: found a number
*****************************************************************/

int OptSearch( char *option )
{
    int           i,j;
    unsigned long secs;
    char          *begin, *ptr;

    /*
     * convert option string to upper case
     */

    ptr = option;
    while ( *ptr )
    {
        if ( 'a' <= *ptr && *ptr <= 'z' )
            *ptr -= 32;
        ++ptr;
    }

    /*
     * locate option
     */

    for ( i = 0; i < MON_MAX; i++ )
    {
        begin = option;
        if ( strcmp( begin, optlist[i] ) == 0 )
        {
            switch( i )
            {
                case MON_MON:
                    for (j=2; j<MON_MAX;j++)
                       mon[j].display = SHOW_ON;
                    break;
                case MON_INFO:
                    mon[ MON_INFO ].display = SHOW_ON;
                    break;
                case MON_MEM:
                    mon[ MON_MEM ].display = SHOW_ON;
                    break;
                case MON_IF:
                    mon[ MON_IF ].display = SHOW_ON;
                    break;
                case MON_IP:
                    mon[ MON_IP ].display = SHOW_ON;
                    break;
                case MON_ICMP:
                    mon[ MON_ICMP ].display = SHOW_ON;
                    break;
                case MON_UDP:
                    mon[ MON_UDP ].display = SHOW_ON;
                    break;
                case MON_UDPCON:
                    mon[ MON_UDPCON ].display = SHOW_ON;
                    break;
                case MON_TCP:
                    mon[ MON_TCP ].display = SHOW_ON;
                    break;
                case MON_TCPCON:
                    mon[ MON_TCPCON ].display = SHOW_ON;
                    break;
            }
            return 0;
        }
    }

    /*
     * try for a number
     */

    displaywait = atol( option );
    if ( displaywait < MIN_PERIOD )
        displaywait = MIN_PERIOD;
    if ( displaywait > MAX_PERIOD )
        displaywait = MAX_PERIOD;
    return -1;
}

/*****************************************************************
** NAME:        MonInit                                     NETMON
** SYNOPSIS:    void MonInit
**              (
**                void
**              )
** RELATED TO:  Mon..(), Sys..()
** DESCRIPTION: initialise mon stats
** REMARKS:     - dummy
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void MonInit( void )
{
}

/*****************************************************************
** NAME:        MonText                                     NETMON
** SYNOPSIS:    void MonText
**              (
**                int x,      relative column origin
**                int y       relative row    origin
**              )
** RELATED TO:  Mon..(), Sys..()
** DESCRIPTION: display mon header texts
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void MonText( int x, int y )
{
    TermValue( montext, x, y, NTEXT );
}

/*****************************************************************
** NAME:        MonUpdate                                   NETMON
** SYNOPSIS:    void MonUpdate
**              (
**                int x,      relative column origin
**                int y       relative row    origin
**              )
** RELATED TO:  Mon..(), Sys..()
** DESCRIPTION: display mon header values
** REMARKS:     - times since Netmon start and display period
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void MonUpdate( int x, int y )
{
    tprintf( x+7, y+1, " % 12.2f", timenew );
    tprintf( x+39, y+1, " % 6.2f", timedone );
}

/* end of netmon.c */

