/************************************************************************
** MODULE INFORMATION **
************************
** FILE NAME:          nettcp.c
** SYSTEM NAME:        Netmon
** TYPE NAME:          TCP
** ORIGINAL AUTHOR(S): L. Th. Weerpas & W. Haverhals
** VERSION NUMBER:     1.0
** CREATION DATE:      1992/6/9
**
** DESCRIPTION:        Internet TCP stat retrieval
*************************************************************************
** CHANGES INFORMATION **
*************************
** REVISION:           $Revision$
** WORKFILE:           $Workfile$
** LOGINFO:            $Log$
*************************************************************************
** COPYRIGHT           (c) 1992
**                     L. Th. Weerpas & W. Haverhals
**                     All rights reserved.
**
** NOTICE              Redistribution  and use in source and binary forms
**                     are  permitted  provided  that the above copyright
**                     notice  and  this  paragraph are duplicated in all
**                     such forms and that any documentation, advertising
**                     materials,  and  other  materials  related to such
**                     distribution and use acknowledge that the software
**                     was  developed  by  above  mentioned  authors. The
**                     name(s)  of the authors may not be used to endorse
**                     or  promote  products  derived  from this software
**                     without  specific  prior  written permission.
**                     THIS  SOFTWARE  IS  PROVIDED ``AS IS'' AND WITHOUT
**                     ANY  EXPRESS  OR  IMPLIED  WARRANTIES,  INCLUDING,
**                     WITHOUT  LIMITATION,  THE  IMPLIED  WARRANTIES  OF
**                     MERCHANTIBILITY   AND  FITNESS  FOR  A  PARTICULAR
**                     PURPOSE.
************************************************************************/

#define _NETTCP_C

#include "netmon.h"
#include "nettcp.h"

/*
 * GLOBALS
 */

static NETTCPSTAT   stat1;
static NETTCPSTAT   stat2;
static NETTCPSTAT  *pold;
static NETTCPSTAT  *pnew;

/*
 * kernel access
 */

static KVMLIST symlist[] =
{
    { "_tcpstat" },
    { NULL }
};
#define NSYM  (int)(sizeof(symlist)/sizeof( KVMLIST ))

static KVMOFST ofst_tcpstat;

/*
 * general
 */

#define CL1  (0)
#define CL1v (CL1+17)
#define CL2  (CL1+30)
#define CL2v (CL2+15)
#define CL3  (CL2+39)
#define CL3v (CL3+15)

MODFUNC modtcp = { TcpInit, TcpText, TcpUpdate, CL3+38, 22 };

MODTEXT tcptext[] =
{
    {  0, CL1 , "TCP_GENERAL______" },
    {  0, CL1v, " #COUNTER__" },
    {  0, CL2 , "TCP_SEND_______" },
    {  0, CL2v, " #COUNT____ PER/SEC___" },
    {  0, CL3 , "TCP_RECEIVE____" },
    {  0, CL3v, " #COUNT____ PER/SEC___" },

    {  1, CL1, "conn. attempts" },
    {  2, CL1, "conn. accepted" },
    {  3, CL1, "conn. established" },
    {  4, CL1, "conn. dropped" },
    {  5, CL1, "embryonic dropped" },
    {  6, CL1, "conn. closed" },
    {  7, CL1, "timing segments" },
    {  8, CL1, "timing updates" },
    {  9, CL1, "delayed segments" },
    { 10, CL1, "conn drop_timeout" },
    { 11, CL1, "rxmit timeout" },
    { 12, CL1, "persist timeout" },
    { 13, CL1, "keepalive timeout" },
    { 14, CL1, "keepalive probes" },
    { 15, CL1, "keepalive drops" },

    {  1, CL2, "send pack total" },
    {  2, CL2, "  data packet" },
    {  3, CL2, "  data bytes" },
    {  4, CL2, "  data pack rx" },
    {  5, CL2, "  data byte rx" },
    {  6, CL2, "  ack only pack" },
    {  7, CL2, "  window probe" },
    {  8, CL2, "  urg only" },
    {  9, CL2, "  win update" },
    { 10, CL2, "  other ctrls" },

    {  1, CL3, "rcvd pack total" },
    {  2, CL3, "  pack in seq." },
    {  3, CL3, "  bytes in seq." },
    {  4, CL3, "  pack bad sum" },
    {  5, CL3, "  pack bad ofs" },
    {  6, CL3, "  pack tooshort" },
    {  7, CL3, "  pack dup.only" },
    {  8, CL3, "  byte dup.only" },
    {  9, CL3, "  pack dup some" },
    { 10, CL3, "  byte dup some" },
    { 11, CL3, "  pack oob" },
    { 12, CL3, "  byte oob" },
    { 13, CL3, "  pack overflow" },
    { 14, CL3, "  byte overflow" },
    { 15, CL3, "  pack after cl" },
    { 16, CL3, "  pack win prob" },
    { 17, CL3, "  pack dup ack" },
    { 18, CL3, "  pack ack2much" },
    { 19, CL3, "  pack w/ack" },
    { 20, CL3, "  byte acked" },
    { 21, CL3, "  pack win updt" }
};
#define NTEXT (sizeof(tcptext)/sizeof(MODTEXT))

/* -----------------------------------------------------------------------------
TCP_GENERAL______ #COUNTER__
conn. attempts    1234567890
conn. accepted    1234567890
conn. established 1234567890
conn. dropped     1234567890
embryonic dropped 1234567890
conn. closed      1234567890
timing segments   1234567890
timing updates    1234567890
delayed segments  1234567890
conn drop_timeout 1234567890
rxmit timeout     1234567890
persist timeout   1234567890
keepalive timeout 1234567890
keepalive probes  1234567890
keepalive drops   1234567890

TCP_SEND_______ #COUNT____ PER/SEC___
send pack total 1234567890 1234567.90
  data packet   1234567890 1234567.90
  data bytes    1234567890 1234567.90
  data pack rx  1234567890 1234567.90
  data byte rx  1234567890 1234567.90
  ack only pack 1234567890 1234567.90
  window probe  1234567890 1234567.90
  urg only      1234567890 1234567.90
  win update    1234567890 1234567.90
  other ctrls   1234567890 1234567.90

TCP_RECEIVE____ #COUNT____ PER/SEC___
rcvd pack total 1234567890 1234567.90
  pack in seq.  1234567890 1234567.90
  bytes in seq. 1234567890 1234567.90
  pack bad sum  1234567890 1234567.90
  pack bad ofs  1234567890 1234567.90
  pack tooshort 1234567890 1234567.90
  pack dup.only 1234567890 1234567.90
  byte dup.only 1234567890 1234567.90
  pack dup some 1234567890 1234567.90
  byte dup some 1234567890 1234567.90
  pack oob      1234567890 1234567.90
  byte oob      1234567890 1234567.90
  pack overflow 1234567890 1234567.90
  byte overflow 1234567890 1234567.90
  pack after cl 1234567890 1234567.90
  pack win prob 1234567890 1234567.90
  pack dup ack  1234567890 1234567.90
  pack ack2much 1234567890 1234567.90
  pack w/ack    1234567890 1234567.90
  byte acked    1234567890 1234567.90
  pack win updt 1234567890 1234567.90
----------------------------------------------------------------------------- */

/*
 * local prototypes
 */

void TcpCnt1( int x, int y, u_long *new );
void TcpCnt2( int x, int y, u_long *new, u_long *old);

#define O1(_dx,_dy,_fld) TcpCnt1(x+_dx,y+_dy, &pnew->_fld)
#define O2(_dx,_dy,_fld) TcpCnt2(x+_dx,y+_dy, &pnew->_fld, &pold->_fld)

/*****************************************************************
** NAME:        TcpInit                                     NETMON
** SYNOPSIS:    void TcpInit
**              (
**                void
**              )
** RELATED TO:  Tcp..()
** DESCRIPTION: initialise tcp stats
** REMARKS:     - kernel access init for TCP stats
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void TcpInit( void )
{
    int r;

    memset( &stat1, 0, sizeof( stat1 ));
    memset( &stat2, 0, sizeof( stat2 ));
    pold = &stat1;
    pnew = &stat2;

    /*
     * locate the wanted symbols
     */

    KvmNlist( symlist, NSYM, __FILE__ );
    ofst_tcpstat = symlist[0].n_value;

    /*
     * initialise the new (is next old!) DB
     */

    KvmRead( ofst_tcpstat, pnew, sizeof( struct tcpstat ));
}

/*****************************************************************
** NAME:        TcpText                                     NETMON
** SYNOPSIS:    void TcpText
**              (
**                int x,      absolute column origin
**                int y       absolute row    origin
**              )
** RELATED TO:  Tcp..()
** DESCRIPTION: display tcp texts
** REMARKS:     - static texts only
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void TcpText( int x, int y )
{
    TermValue( tcptext, x, y, NTEXT );
}

/*****************************************************************
** NAME:        TcpUpdate                                   NETMON
** SYNOPSIS:    void TcpUpdate
**              (
**                int x,      absolute column origin
**                int y       absolute row    origin
**              )
** RELATED TO:  Tcp..()
** DESCRIPTION: display tcp dynamic vallues
** REMARKS:     - called by SysUpdate()
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void TcpUpdate( int x, int y )
{
    u_short     sent;
    NETTCPSTAT *swap;

    swap = pold;
    pold = pnew;
    pnew = swap;

    KvmRead( ofst_tcpstat, pnew, sizeof( struct tcpstat ));

    O1( CL1v,  1, tcps_connattempt );
    O1( CL1v,  2, tcps_accepts );
    O1( CL1v,  3, tcps_connects );
    O1( CL1v,  4, tcps_drops );
    O1( CL1v,  5, tcps_conndrops );
    O1( CL1v,  6, tcps_closed );
    O1( CL1v,  7, tcps_segstimed );
    O1( CL1v,  8, tcps_rttupdated );
    O1( CL1v,  9, tcps_delack );
    O1( CL1v, 10, tcps_timeoutdrop );
    O1( CL1v, 11, tcps_rexmttimeo );
    O1( CL1v, 12, tcps_persisttimeo );
    O1( CL1v, 13, tcps_keeptimeo );
    O1( CL1v, 14, tcps_keepprobe );
    O1( CL1v, 15, tcps_keepdrops );

    O2( CL2v,  1, tcps_sndtotal );
    O2( CL2v,  2, tcps_sndpack );
    O2( CL2v,  3, tcps_sndbyte );
    O2( CL2v,  4, tcps_sndrexmitpack );
    O2( CL2v,  5, tcps_sndrexmitbyte );
    O2( CL2v,  6, tcps_sndacks );
    O2( CL2v,  7, tcps_sndprobe );
    O2( CL2v,  8, tcps_sndurg );
    O2( CL2v,  9, tcps_sndwinup );
    O2( CL2v, 10, tcps_sndctrl );

    O2( CL3v,  1, tcps_rcvtotal );
    O2( CL3v,  2, tcps_rcvpack );
    O2( CL3v,  3, tcps_rcvbyte );
    O2( CL3v,  4, tcps_rcvbadsum );
    O2( CL3v,  5, tcps_rcvbadoff );
    O2( CL3v,  6, tcps_rcvshort );
    O2( CL3v,  7, tcps_rcvduppack );
    O2( CL3v,  8, tcps_rcvdupbyte );
    O2( CL3v,  9, tcps_rcvpartduppack );
    O2( CL3v, 10, tcps_rcvpartdupbyte );
    O2( CL3v, 11, tcps_rcvoopack );
    O2( CL3v, 12, tcps_rcvoobyte );
    O2( CL3v, 13, tcps_rcvpackafterwin );
    O2( CL3v, 14, tcps_rcvbyteafterwin );
    O2( CL3v, 15, tcps_rcvafterclose );
    O2( CL3v, 16, tcps_rcvwinprobe );
    O2( CL3v, 17, tcps_rcvdupack );
    O2( CL3v, 18, tcps_rcvacktoomuch );
    O2( CL3v, 19, tcps_rcvackpack );
    O2( CL3v, 20, tcps_rcvackbyte );
    O2( CL3v, 21, tcps_rcvwinupd );
}

/*****************************************************************
** NAME:        TcpCnt1                                     NETMON
** SYNOPSIS:    void TcpCnt1
**              (
**                int x,      absolute column origin
**                int y       absolute row    origin
**                u_long *new ptr to value
**              )
** RELATED TO:  Tcp..()
** DESCRIPTION: output TCP stat counts:value
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void TcpCnt1( int x, int y, u_long *new )
{
    tprintf( x, y, " % 10lu", *new );
}

/*****************************************************************
** NAME:        TcpCnt2                                     NETMON
** SYNOPSIS:    void TcpCnt2
**              (
**                int x,       absolute column origin
**                int y,       absolute row    origin
**                u_long *new, ptr to last value
**                u_long *old  ptr to previous value
**              )
** RELATED TO:  Tcp..()
** DESCRIPTION: output TCP stat counts:value,rate
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void TcpCnt2( int x, int y, u_long *new, u_long *old)
{
    u_long diff;

    if (*new < *old )
        diff = *old - *new;
    else
        diff = *new - *old;

    tprintf( x, y, " % 10lu % 10.2f",
                   *new,
                   (double) ((double) (diff)/timedone));
}

/* end of nettcp.c */



