/************************************************************************
** MODULE INFORMATION **
************************
** FILE NAME:          netudp.c
** SYSTEM NAME:        Netmon
** TYPE NAME:          UDP
** ORIGINAL AUTHOR(S): L. Th. Weerpas & W. Haverhals
** VERSION NUMBER:     1.0
** CREATION DATE:      1992/6/9
**
** DESCRIPTION:        Internet UDP stat retrieval
**                     The kernel offers rather uninteresting data !
*************************************************************************
** CHANGES INFORMATION **
*************************
** REVISION:           $Revision$
** WORKFILE:           $Workfile$
** LOGINFO:            $Log$
*************************************************************************
** COPYRIGHT           (c) 1992
**                     L. Th. Weerpas & W. Haverhals
**                     All rights reserved.
**
** NOTICE              Redistribution  and use in source and binary forms
**                     are  permitted  provided  that the above copyright
**                     notice  and  this  paragraph are duplicated in all
**                     such forms and that any documentation, advertising
**                     materials,  and  other  materials  related to such
**                     distribution and use acknowledge that the software
**                     was  developed  by  above  mentioned  authors. The
**                     name(s)  of the authors may not be used to endorse
**                     or  promote  products  derived  from this software
**                     without  specific  prior  written permission.
**                     THIS  SOFTWARE  IS  PROVIDED ``AS IS'' AND WITHOUT
**                     ANY  EXPRESS  OR  IMPLIED  WARRANTIES,  INCLUDING,
**                     WITHOUT  LIMITATION,  THE  IMPLIED  WARRANTIES  OF
**                     MERCHANTIBILITY   AND  FITNESS  FOR  A  PARTICULAR
**                     PURPOSE.
************************************************************************/

#define _NETUDP_C

#include "netmon.h"
#include "netudp.h"

/*
 * GLOBALS
 */

static NETUDPSTAT   stat1;
static NETUDPSTAT   stat2;
static NETUDPSTAT  *pold;
static NETUDPSTAT  *pnew;

/*
 * kernel access
 */

static KVMLIST symlist[] =
{
    { "_udpstat" },      /* some stats */
    { NULL }
};
#define NSYM  (int)(sizeof(symlist)/sizeof( KVMLIST ))

static KVMOFST ofst_udpstat;

MODFUNC modudp = { UdpInit, UdpText, UdpUpdate, 40, 5 };
MODTEXT udptext[] =
{
    { 0,  0, "PROTOCOL: UDP" },
    { 1,  0, "header ops" },
    { 2,  0, "bad sum" },
    { 3,  0, "bad length" },
    { 4,  0, "full sockets" }
};
#define NTEXT (sizeof(udptext)/sizeof(MODTEXT))

/*
 * local prototypes
 */

void UdpCnt1( int x, int y, u_long *new );
void UdpCnt2( int x, int y, u_long *new, u_long *old);

#define O1(_dx,_dy,_fld) UdpCnt1(x+_dx,y+_dy, &pnew->_fld)
#define O2(_dx,_dy,_fld) UdpCnt2(x+_dx,y+_dy, &pnew->_fld, &pold->_fld)

/*****************************************************************
** NAME:        UdpInit                                     NETMON
** SYNOPSIS:    void UdpInit
**              (
**                void
**              )
** RELATED TO:  Udp..()
** DESCRIPTION: initialise udp stats
** REMARKS:     - locate udp kernel vars
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void UdpInit( void )
{
    int r;

    memset( &stat1, 0, sizeof( stat1 ));
    memset( &stat2, 0, sizeof( stat2 ));
    pold = &stat1;
    pnew = &stat2;

    /*
     * locate the wanted symbols
     */

    KvmNlist( symlist, NSYM, __FILE__ );
    ofst_udpstat = symlist[0].n_value;

    /*
     * initialise the new (is next old!) DB
     */

    KvmRead( ofst_udpstat, &pnew->stat, sizeof( struct udpstat ));
}

/*****************************************************************
** NAME:        UdpText                                     NETMON
** SYNOPSIS:    void UdpText
**              (
**                int x,      absolute column origin
**                int y       absolute row    origin
**              )
** RELATED TO:  Udp..()
** DESCRIPTION: print static UDP texts
** REMARKS:     - display udp texts
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void UdpText( int x, int y )
{
    TermValue( udptext, x, y, NTEXT );
}

/*****************************************************************
** NAME:        UdpUpdate                                   NETMON
** SYNOPSIS:    void UdpUpdate
**              (
**                int x,      absolute column origin
**                int y       absolute row    origin
**              )
** RELATED TO:  Udp..()
** DESCRIPTION: display udp values and update stats
** REMARKS:     - reads the kernel
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void UdpUpdate( int x, int y )
{
    u_short     sent;
    NETUDPSTAT *swap;

    swap = pold;
    pold = pnew;
    pnew = swap;

    KvmRead( ofst_udpstat, &pnew->stat, sizeof( struct udpstat ));

    O2( 14, 1, stat.udps_hdrops );
    O2( 14, 2, stat.udps_badsum );
    O2( 14, 3, stat.udps_badlen );
    O2( 14, 4, stat.udps_fullsock );
}

/*****************************************************************
** NAME:        UdpCnt1                                     NETMON
** SYNOPSIS:    void UdpCnt1
**              (
**                int x,      absolute column origin
**                int y,      absolute row    origin
**                u_long *new pointer to value
**              )
** RELATED TO:  Udp..()
** DESCRIPTION: output UDP stat counts:value
** REMARKS:     - output formatter
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void UdpCnt1( int x, int y, u_long *new )
{
    tprintf( x, y, " % 9lu", *new );
}

/*****************************************************************
** NAME:        UdpCnt2                                     NETMON
** SYNOPSIS:    void UdpCnt2
**              (
**                int x,        absolute column origin
**                int y,        absolute row    origin
**                u_long *new,  pointer to new value
**                u_long *old   pointer to old value
**              )
** RELATED TO:  Udp..()
** DESCRIPTION: output UDP stat counts:value,rate
** REMARKS:     - output formatter
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void UdpCnt2( int x, int y, u_long *new, u_long *old)
{
    u_long diff;

    if (*new < *old )
        diff = *old - *new;
    else
        diff = *new - *old;

    tprintf( x, y, " % 9lu % 9.2f",
                   *new,
                   (double) ((double) (diff)/timedone));
}

/* end of netudp.c */




