/************************************************************************
** MODULE INFORMATION **
************************
** FILE NAME:          netudpso.c
** SYSTEM NAME:        Netmon
** TYPE NAME:          network UDP sockets
** ORIGINAL AUTHOR(S): L. Th. Weerpas & W. Haverhals
** VERSION NUMBER:     1.0
** CREATION DATE:      1992/6/14
**
** DESCRIPTION:        produce running view of UDP sockets and users
*************************************************************************
** CHANGES INFORMATION **
*************************
** REVISION:           $Revision$
** WORKFILE:           $Workfile$
** LOGINFO:            $Log$
*************************************************************************
** COPYRIGHT           (c) 1992
**                     L. Th. Weerpas & W. Haverhals
**                     All rights reserved.
**
** NOTICE              Redistribution  and use in source and binary forms
**                     are  permitted  provided  that the above copyright
**                     notice  and  this  paragraph are duplicated in all
**                     such forms and that any documentation, advertising
**                     materials,  and  other  materials  related to such
**                     distribution and use acknowledge that the software
**                     was  developed  by  above  mentioned  authors. The
**                     name(s)  of the authors may not be used to endorse
**                     or  promote  products  derived  from this software
**                     without  specific  prior  written permission.
**                     THIS  SOFTWARE  IS  PROVIDED ``AS IS'' AND WITHOUT
**                     ANY  EXPRESS  OR  IMPLIED  WARRANTIES,  INCLUDING,
**                     WITHOUT  LIMITATION,  THE  IMPLIED  WARRANTIES  OF
**                     MERCHANTIBILITY   AND  FITNESS  FOR  A  PARTICULAR
**                     PURPOSE.
************************************************************************/

#define _NETUDPSO_C

#include "netmon.h"
#include "netudpso.h"

/*
 * GLOBALS
 */

/*
 * kernel access
 */

static KVMLIST symlist[] =
{
    { "_udb" },          /* protocol control block */
    { NULL }
};
#define NSYM  (int)(sizeof(symlist)/sizeof( KVMLIST ))

static KVMOFST ofst_udb;

/* texts */

static char header[] = "FOREIGN_ADDRESS,PORT___  "
                       "__LOCAL_ADDRESS,PORT___  "
                       "SOCKET_STATE____  "
                       "SOCKET_OPTIONS__  "
                       "RBFmx_  SBFmx_  "
                       "RBF___  SBF___  "
                       "USERNAME__  PROCESS______udp";
#define NHDR (sizeof(header))

MODFUNC modudpso = { UdpsoInit, UdpsoText, UdpsoUpdate, NHDR+1, MTS+1 };
MODTEXT udpsotext[] =
{
    { 0,  0, header }
};
#define NTEXT (sizeof(udpsotext)/sizeof(MODTEXT))

/*
 * available inpcb's
 */

static short       npcbs;               /* nr of inpcb's for UDP */
static NETUDPSO    udpso[ MAX_SO ];
static struct user usr;                 /* temp user area        */
static char        username[ 256 ];
static char        processname[ 256 ];

/*
 * local prototypes
 */

void UdpsoSockInfo( NETUDPSO *ptis );

/*****************************************************************
** NAME:        UdpsoInit                                   NETMON
** SYNOPSIS:    void UdpsoInit
**              (
**                void
**              )
** RELATED TO:  Udpso..()
** DESCRIPTION: initialise udpso stats
** REMARKS:     - init kernel read only.
** RETURN TYPE: none
** EXIT STATUS: -
*****************************************************************/

void UdpsoInit( void )
{
    memset( udpso, 0, sizeof( udpso ));

    /*
     * locate the wanted symbols
     */

    KvmNlist( symlist, NSYM, __FILE__ );
    ofst_udb     = symlist[0].n_value;
    return;
}

/*****************************************************************
** NAME:        UdpsoText                                   NETMON
** SYNOPSIS:    void UdpsoText
**              (
**                int x,      column nr
**                int y       row    nr
**              )
** RELATED TO:  Udpso..()
** DESCRIPTION: display udpso texts
** REMARKS:     -
** RETURN TYPE: none
** EXIT STATUS: -
*****************************************************************/

void UdpsoText( int x, int y )
{
    TermValue( udpsotext, x, y, NTEXT );
}

/*****************************************************************
** NAME:        UdpSoUpdate                                 NETMON
** SYNOPSIS:    void UdpsoUpdate
**              (
**                int x,      column nr
**                int y       row    nr
**              )
** RELATED TO:  Udpso..()
** DESCRIPTION: display udpso values
** REMARKS:     -
** RETURN TYPE: none
** EXIT STATUS: -
*****************************************************************/

void UdpsoUpdate( int x, int y )
{
    int       r;              /* result of KvmRead2() */
    int       row;
    int       cnt;
    int       s, i;

    KVMOFST   ofst_inpcb;
    KVMOFST   ofst_sock;
    KVMOFST   ofst_rprc;
    KVMOFST   ofst_sprc;
    NETUDPSO *ptis;
    INPCB    *pin;
    u_long    foreignadr;
    u_long    localadr;

    /*
     * Run  this  routine  on  a very high priority level. We
     * must and I mean MUST ensure that the network subsystem
     * DOES  NOT change the linked lists while we are reading
     * them.   This  could  cause  incorrect  data  and  also
     * segmentation violations.
     */

    s = spl3();

    /*
     * Retrieve all the data structures. We only retrieve UDP
     * connections with existing SOCKETs.
     * The _ucb list seems to be a circular double linked list.
     * Therefor the end criterium is   ofst_inpcb == ofst_ucb
     */

    ofst_inpcb = ofst_udb;                     /* first entry offset */

    for ( npcbs=0; npcbs < MAX_SO; )
    {
        ptis = &udpso[ npcbs ];

        /*
         * read INPCB :
         * 'udb' is a struct and thus always present
         */

        r = KvmRead2( ofst_inpcb, &ptis->ipcb, sizeof( INPCB ));
        if ( r < 0 )
            break;

        /*
         * read SOCKET : possibly present
         * if not : advance to the next inpcb
         */

        ofst_sock = (KVMOFST) ptis->ipcb.inp_socket;
        if ( ofst_sock )
        {
            r = KvmRead2( ofst_sock, &ptis->sock, sizeof( SOCKET ));
            if ( r < 0 )
                goto L0;
            ptis->sockok = VALID;

            /*
             * read receiving PROCESS
             */

            ofst_rprc = (KVMOFST) ptis->sock.so_rcv.sb_sel;
            r = KvmRead2( ofst_rprc, &ptis->rprc, sizeof( PROCESS ));
            if ( r >= 0 )
                ptis->rprocok = VALID;

            /*
             * read sending PROCESS
             */

            ofst_sprc = (KVMOFST) ptis->sock.so_snd.sb_sel;
            r = KvmRead2( ofst_sprc, &ptis->sprc, sizeof( PROCESS ));
            if ( r >= 0 )
                ptis->sprocok = VALID;
        }
        else
            goto L0;

        /*
         * done, to the next inpcb and check on wrap around
         */

        ++npcbs;
L0:     ofst_inpcb = (KVMOFST) ptis->ipcb.inp_next;
        if ( ofst_inpcb == ofst_udb )
            break;
    }

    /*
     * reset the priority
     */

    splx( s );

    /*
     * update the terminal screen
     */

    row = y+1;
    cnt = MTS;

    /*
     * fully specified sockets
     */

    for ( i = 0; i < npcbs && cnt ; i++ )
    {
        /*
         * set the pointers
         */

        ptis = &udpso[ i ];
        pin  = &ptis->ipcb;

        /*
         * 46   connection of this inpcb
         */

        foreignadr = ntohl( pin->inp_faddr.s_addr );
        localadr   = ntohl( pin->inp_laddr.s_addr );
        if ( localadr != 0 && foreignadr != 0 )
        {
            TermSetXY( x, row++ );
            SockAdrPortOut( &pin->inp_faddr, pin->inp_fport, NET_UDP );
            SockAdrPortOut( &pin->inp_laddr, pin->inp_lport, NET_UDP );

            /*
             * 18,32  socket state
             */

            UdpsoSockInfo( ptis );

            /*
             *>96    change linecount
             */

            cnt--;
        }
    }

    /*
     * locally specified sockets
     */

    for ( i = 0; i < npcbs && cnt ; i++ )
    {
        /*
         * set the pointers
         */

        ptis = &udpso[ i ];
        pin  = &ptis->ipcb;

        /*
         * 46   connection of this inpcb
         */

        foreignadr = ntohl( pin->inp_faddr.s_addr );
        localadr   = ntohl( pin->inp_laddr.s_addr );
        if ( localadr != 0 && foreignadr == 0 )
        {
            TermSetXY( x, row++ );
            SockAdrPortOut( &pin->inp_faddr, pin->inp_fport, NET_UDP );
            SockAdrPortOut( &pin->inp_laddr, pin->inp_lport, NET_UDP );

            /*
             * 18,32  socket state
             */

            UdpsoSockInfo( ptis );

            /*
             *>96    change linecount
             */

            cnt--;
        }
    }

    /*
     * unbounded sockets
     */

    for ( i = 0; i < npcbs && cnt ; i++ )
    {
        /*
         * set the pointers
         */

        ptis = &udpso[ i ];
        pin  = &ptis->ipcb;

        /*
         * 46   connection of this inpcb
         */

        foreignadr = ntohl( pin->inp_faddr.s_addr );
        localadr   = ntohl( pin->inp_laddr.s_addr );
        if ( localadr == 0 && foreignadr == 0 )
        {
            TermSetXY( x, row++ );
            SockAdrPortOut( &pin->inp_faddr, pin->inp_fport, NET_UDP );
            SockAdrPortOut( &pin->inp_laddr, pin->inp_lport, NET_UDP );

            /*
             * 18,32  socket state
             */

            UdpsoSockInfo( ptis );

            /*
             *>96    change linecount
             */

            cnt--;
        }
    }

    /*
     * clear the terminal to end of screen
     */

    TermClearToEnd();
    return;
}

/*****************************************************************
** NAME:        UdpsoSockInfo                               NETMON
** SYNOPSIS:    void UdpsoSockInfo
**              (
**                NETUDPSO *ptis
**              )
** RELATED TO:  Udpso..()
** DESCRIPTION: UDP socket: display user and process
** REMARKS:     -
** RETURN TYPE: none
** EXIT STATUS: -
*****************************************************************/

void UdpsoSockInfo( NETUDPSO *ptis )
{
    if ( ptis->sockok == VALID )
    {
        SockStateInfo( ptis->sock.so_state );
        SockOptions( &ptis->sock );
        SockBufSize( &ptis->sock );
        if ( ptis->rprocok == VALID )
        {
             cprintf( "%10.10s  %16.16s",
                      GetUserName( ptis->rprc.p_uid, username ),
                      GetProcName( &ptis->rprc, processname ) );
        }
        else if ( ptis->sprocok == VALID )
        {
             cprintf( "%10.10s  %16.16s",
                      GetUserName( ptis->sprc.p_uid, username ),
                      GetProcName( &ptis->sprc, processname ) );
        }
        else
        {
            cprintf( "---user?--  ----process?----" );
        }
    }
    else
    {
        cprintf( "---- socket? -----" );
        cprintf( "---user?--  ----process?----" );
    }
}

/* end of netudpso.c */


